/* (C) 2022 YiRing, Inc. */
package com.yiring.app.job;

import com.alibaba.fastjson.JSON;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.yiring.app.design.strategy.rule.ScanAlarmContext;
import com.yiring.app.domain.location.LocationFence;
import com.yiring.app.domain.location.LocationFenceRepository;
import com.yiring.app.domain.location.LocationFenceRule;
import com.yiring.app.param.location.fence.LocationFenceJobParam;
import com.yiring.app.param.location.rule.RuleParam;
import com.yiring.app.vo.location.fence.LocationFenceJobVo;
import java.util.List;
import java.util.Set;
import javax.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

/**
 * @author tml
 * @version 1.0
 * @date 2022/5/14 10:10
 */
@Slf4j
@Component
public class FenceAlarmJob {

    @Resource
    private LocationFenceRepository locationFenceRepository;

    @XxlJob("ScanAlarmHandler")
    public void scanAlarmHandler() {
        //查询出所有已启用的电子围栏信息
        List<LocationFence> fenceList = locationFenceRepository.findAllByEnable(true);
        for (LocationFence fence : fenceList) {
            //获取该围栏下的所有报警规则
            Set<LocationFenceRule> fenceRuleSet = fence.getRules();
            for (LocationFenceRule fenceRule : fenceRuleSet) {
                RuleParam ruleParam = JSON.parseObject(fenceRule.getRule(), RuleParam.class);
                LocationFenceJobParam fenceJobParam = LocationFenceJobParam
                    .builder()
                    .rule(ruleParam)
                    .fenceId(fence.getId())
                    .alarmTypeId(fenceRule.getAlarmType().getId())
                    .tagSet(fence.getTags())
                    .mapId(fence.getMapId())
                    //todo 查询出正在报警的规则
                    //.oldAlarms()
                    .build();
                LocationFenceJobVo locationFenceJobVo = ScanAlarmContext.scanAlarm(
                    fenceRule.getRelevanceParamType(),
                    fenceJobParam
                );
                //todo 将结果入库
            }
        }
    }
}
