/* (C) 2022 YiRing, Inc. */
package com.yiring.app.rabbit.config;

import com.yiring.app.rabbit.receiver.PositionMessageHandler;
import javax.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.AcknowledgeMode;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * 真源 RabbitMQ 订阅配置
 *
 * @author Jim
 * @version 0.1
 * 2022/4/13 17:05
 */

@Slf4j
@Configuration
@ConditionalOnProperty(prefix = "zy-config.rabbitmq", value = "enabled", havingValue = "true")
public class ZyRabbitConfig {

    @Resource
    ZyConfigProperties.ZyConfigRabbitmq rabbitmq;

    public static final String CONNECTION_FACTORY_NAME = "zyRabbitConnectionFactory";
    public static final String LISTENER_CONTAINER_NAME = "zyPositionMessageListenerContainer";
    public static final String QUEUE_BEAN_NAME = "messageHandlerQueue";

    @Bean(CONNECTION_FACTORY_NAME)
    public ConnectionFactory zyConnectionFactory() {
        return RabbitConfig.connectionFactory(
            rabbitmq.getHost(),
            rabbitmq.getPort(),
            rabbitmq.getUsername(),
            rabbitmq.getPassword(),
            rabbitmq.getVirtualHost()
        );
    }

    @Bean(LISTENER_CONTAINER_NAME)
    @ConditionalOnProperty(prefix = "zy-config.rabbitmq", value = "mock", havingValue = "false")
    public SimpleMessageListenerContainer simpleMessageListenerContainer(
        PositionMessageHandler handler,
        @Qualifier(QUEUE_BEAN_NAME) Queue queue,
        @Qualifier(CONNECTION_FACTORY_NAME) ConnectionFactory connectionFactory
    ) {
        return buildPositionMessageListenerContainer(handler, queue, connectionFactory);
    }

    @Bean(QUEUE_BEAN_NAME)
    public Queue messageHandlerQueue() {
        return new Queue(rabbitmq.getQueueName(), true, false, false);
    }

    public static SimpleMessageListenerContainer buildPositionMessageListenerContainer(
        PositionMessageHandler handler,
        Queue queue,
        ConnectionFactory connectionFactory
    ) {
        SimpleMessageListenerContainer container = new SimpleMessageListenerContainer();
        container.setConnectionFactory(connectionFactory);
        container.setAcknowledgeMode(AcknowledgeMode.MANUAL);
        container.setBatchSize(10);
        container.setConcurrentConsumers(1);
        container.setMaxConcurrentConsumers(1);
        container.setExposeListenerChannel(true);
        container.setMessageListener(handler);
        container.addQueues(queue);
        return container;
    }
}
